#include "vbcircum.h"

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


VBCIRCUM::VBCIRCUM (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hHawk = oapiLoadMeshGlobal( "vbcircumlunar");
	tex   = oapiRegisterExhaustTexture ("RealExhaust");
}

void VBCIRCUM::LoadState (FILEHANDLE scn, void *vs)
{
	char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
  		if (!strnicmp (line, "PAYLOAD", 7))
		{
			ParsePayloadString( line );
		}



		else if ( !strnicmp(line, "STAGE",5))
		{
		   ParsePayloadString( line );
		   
		}


		else 
		{
			ParseScenarioLineEx (line, vs);
		}

	}

}

void VBCIRCUM::SaveState (FILEHANDLE scn)
{
	SaveDefaultState (scn);
}

void VBCIRCUM::SetMainStage ( )
{

	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hHawk);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx( tmp);
    Draw();
}

void VBCIRCUM::SetClassCaps (FILEHANDLE cfg)
{

	double MainThrust = 550000;
	double HoverThrust = 2e6;
	double AttThrust = 30000;
	double Ispmain = 2930;
	SPIN_SPEED = 0.0454545;
	spin_proc = 0.0;

	PARTICLESTREAMSPEC contrail = 
	{
		0, 5.0, 5, 250, 0.3, 4.0, 16, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 2.8, 40, 250, 0.1, 0.2, 42, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetRotDrag (_V(0.43,0.43,0.29)); // angular drag


	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	dock0 = CreateDock (_V( 0.0, 0.0 , 10.5), _V(0, 0, 1), _V(0,1, 0));

	SetSize (16);
	SetTouchdownPoints (_V(0,-8.0,6), _V(-2,-8.0,-6), _V(2,-8.0,-6));
	SetPMI (_V(84,77,32));
	SetCrossSections (_V(44,75,15));
	SetCW (0.13, 0.5, 1.4, 1.4);
	SetRotDrag (_V(1.5,1.0,1.0));
	ph_main = CreatePropellantResource( 320000.0 );
	SetDefaultPropellantResource( ph_main );
	SetEmptyMass( 22500.0 + CalculateMass() );
	VECTOR3 pos = _V( 0,0,0 );
//	AddMesh( hHawk, &pos );

	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hHawk);
	tmp.MeshPosition( pos);
	AddMeshEx( tmp);
    Draw();

// 5 Dock #3
/*
	static UINT SpinGrp[5] = {0, 1, 2, 3, 6};
	static MGROUP_ROTATE Spin (0, SpinGrp, 5,
		_V(-0.1,0.0, 0.0), _V(0,0,1), (float)(360*RAD));
	anim_spin = CreateAnimation (0.0);
	AddAnimationComponent (anim_spin, 0.0, 1.0, &Spin);
*/


	th_main[0] = CreateThruster (_V( 0.0, 1.3,-12.7),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, Ispmain, Ispmain);
	th_main[1] = CreateThruster (_V( 0.0,-1.3,-12.7),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, Ispmain, Ispmain);
	th_main[2] = CreateThruster (_V( 1.3, 0.0,-12.7),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, Ispmain, Ispmain);
	th_main[3] = CreateThruster (_V(-1.3, 0.0,-12.7),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, Ispmain, Ispmain);


	thg_main = CreateThrusterGroup( th_main, 4, THGROUP_MAIN );
	AddExhaustStream (th_main[0], _V( 0.0, 1.3,-12.7), &exhaust_main);
	AddExhaustStream (th_main[1], _V( 0.0,-1.3,-12.7), &exhaust_main);
	AddExhaustStream (th_main[2], _V( 1.3, 0.0,-12.7), &exhaust_main);
	AddExhaustStream (th_main[3], _V(-1.3, 0.0,-12.7), &exhaust_main);

	AddExhaust (th_main[0], 10.0, 1.1, tex);
	AddExhaust (th_main[1], 10.0, 1.1, tex);
	AddExhaust (th_main[2], 10.0, 1.1, tex);
	AddExhaust (th_main[3], 10.0, 1.1, tex);
/*
	th_hover[0] = CreateThruster (_V(0,0,0), _V(0,1,0), HoverThrust, ph_main, 4e4, 4e5);
	thg_hover = CreateThrusterGroup (th_hover, 1, THGROUP_HOVER);

	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
*/
	// set of attitude thrusters (idealised). The arrangement is such that no angular
	// momentum is created in linear mode, and no linear momentum is created in rotational mode.
	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,-1.9, 8.5), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[1] = CreateThruster (_V(0, 2.1,-8.5), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[2] = CreateThruster (_V(0, 1.9, 8.5), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[3] = CreateThruster (_V(0,-2.1,-8.5), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);
	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V(0,-1.9, 8.5), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[3] = CreateThruster (_V(0, 2.1,-8.5), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[2] = CreateThruster (_V(0, 1.9, 8.5), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[1] = CreateThruster (_V(0,-2.1,-8.5), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);
	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);

	th_att_rot[0] = CreateThruster (_V( 1.9,0, 8.5), _V(-1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[1] = CreateThruster (_V(-2.1,0,-8.5), _V( 1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[2] = CreateThruster (_V(-1.9,0, 8.5), _V( 1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[3] = CreateThruster (_V( 2.1,0,-8.5), _V(-1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);
	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V( 1.9,0, 8.5), _V(-1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[3] = CreateThruster (_V(-2.1,0,-8.5), _V( 1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[2] = CreateThruster (_V(-1.9,0, 8.5), _V( 1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[1] = CreateThruster (_V( 2.1,0,-8.5), _V(-1,0,0), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);
	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);

	th_att_rot[0] = CreateThruster (_V( 2.8,0,0), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[1] = CreateThruster (_V(-2.8,0,0), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[2] = CreateThruster (_V(-2.8,0,0), _V(0, 1,0), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_rot[3] = CreateThruster (_V( 2.8,0,0), _V(0,-1,0), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);
	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V(   0,0,-9.9), _V(0,0, 1), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[1] = CreateThruster (_V(   0,0,-9.9), _V(0,0, 1), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[2] = CreateThruster (_V(-1.9,0, 8.5), _V(0,0,-1), AttThrust, ph_main, Ispmain, Ispmain);
	th_att_lin[3] = CreateThruster (_V( 1.9,0, 8.5), _V(0,0,-1), AttThrust, ph_main, Ispmain, Ispmain);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_BACK);
	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);
}


void VBCIRCUM::TimeStep( double simt )
{

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() )
		SetMainStage();
}

int VBCIRCUM::ConsumeKey ( const char *keystate)
{
	return 0;
}

int VBCIRCUM::ConsumeBufferedKey ( DWORD dwKey, bool fPressed, const char *keystate)
{

		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
			//	OperateDoors (door_status == DOOR_CLOSED || door_status == DOOR_CLOSING ?
			//		DOOR_OPENING : DOOR_CLOSING);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
					VESSELSTATUS vs;
					OBJHANDLE hvessel;
					GetStatus (vs);
					vs.eng_main = vs.eng_hovr = 0.0;
					vs.fuel=1.0;
//					vs.status=0;
					VECTOR3 ofs = _V(1.1,1.1,5.75);
					Local2Rel (ofs, vs.rpos);
					char name[256], name2[256];

					UINT i;
					for (i=1;i<12;i++) {
							strcpy (name2, GetName()); 
							sprintf(name, "-%d", i);
							strcat (name2, name);
							hvessel=oapiGetVesselByName(name2);
							if (hvessel == 0) break;
					}
	
					VECTOR3 vel = _V(0,0.0,0.0);
					VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
					Local2Rel (ofs, vs.rpos);
					GlobalRot (vel, rofs);
					vs.rvel.x = rvel.x+rofs.x;
					vs.rvel.y = rvel.y+rofs.y;
					vs.rvel.z = rvel.z+rofs.z;
					
					if ( i < 11) {
						VESSEL::Create (name2, "vbsuit", vs);
						hvessel=oapiGetVesselByName(name2);
						oapiSetFocusObject(hvessel);
					}
			}
		return 1;
		}


		return 0;


}




/* basic vessel class setup */

void VBCIRCUM::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void VBCIRCUM::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void VBCIRCUM::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void VBCIRCUM::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void VBCIRCUM::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void VBCIRCUM::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int VBCIRCUM::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VBCIRCUM (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (VBCIRCUM*)vessel;
}

