#include "vbs1x.h"

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


VBS1X::VBS1X (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hHawk = oapiLoadMeshGlobal( "vbFerryS1x");
	tex   = oapiRegisterExhaustTexture ("Exhaust");
}

void VBS1X::LoadState (FILEHANDLE scn, void *vs)
{
	char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
  		if (!strnicmp (line, "PAYLOAD", 7))
		{
			ParsePayloadString( line );
		}



		else if ( !strnicmp(line, "STAGE",5))
		{
		   ParsePayloadString( line );
		   
		}


		else 
		{
			ParseScenarioLineEx (line, vs);
		}

	}

}

void VBS1X::SaveState (FILEHANDLE scn)
{
	SaveDefaultState (scn);
}


void VBS1X::SetClassCaps (FILEHANDLE cfg)
{

	double MainThrust = 3.5e6;
	double HoverThrust = 2e6;
	double RetroThrust = 3e7;
	double AttThrust = 5e6;
	EXPAND_SPEED = 0.5;
	expand_proc = 0.0;
	numevas = 0;

	PARTICLESTREAMSPEC contrail = 
	{
		0, 5.0, 5, 250, 0.3, 0.5, 16, 2.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_retro = {
		0, 2.8, 40, 250, 0.1, 0.2, 42, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_retro.tex = oapiRegisterParticleTexture ("Contrail2");
	SetRotDrag (_V(0.7,0.7,0.3)); // angular drag
//	SetEnableFocus(FALSE);

/*
	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
*/

	SetSize (35);
	SetTouchdownPoints (_V(0,17,12), _V(-10,-10,12), _V(10,10,12));
	SetPMI (_V(38.8, 38.8, 21.5));
	SetCrossSections (_V(64.2, 62.2, 1236.3));
	SetCW (1.3, 1.3, 1.3, 1.3);
	SetRotDrag (_V(0.5, 0.5, 0.3));
	SetPitchMomentScale (5e-4);
	SetBankMomentScale (5e-4);
	ph_main = CreatePropellantResource( 43000.0 );
	SetDefaultPropellantResource( ph_main );

	SetEmptyMass( 770000.0 );
	VECTOR3 pos = _V( 0,0,0 );
//	AddMesh( hHawk, &pos );

	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hHawk);
	tmp.MeshPosition( pos);
	AddMeshEx( tmp);
    Draw();

// 5 Dock #3

	static UINT ExpandGrp[2] = {9, 10};
	static MGROUP_SCALE Expand (0, ExpandGrp, 2,
		_V( 0.0, 0.0, 0.0), _V(2.3,2.3,1.5));
	anim_expand = CreateAnimation (1.0);
	AddAnimationComponent (anim_expand, 0.1, 0.9, &Expand);

	retro_engine[0] = CreateThruster (_V(0,0.0,24.5),    _V( 0.0,0.0,-1.0), 
		RetroThrust, ph_main, 2500, 2500);
//	AddExhaust(retro_engine[0],10.0,2.2, _V(0,0,24.5), _V(0,0,1));
	thg_retro = CreateThrusterGroup(retro_engine,1,THGROUP_RETRO);
	AddExhaustStream (retro_engine[0], _V(0,0,24.5), &contrail);
	int numeng = 18;
	for (int i = 0; i < numeng; i++){
		AddExhaust(retro_engine[0],3.0,0.5, _V((6.9*cos(i*(6.28/numeng))),(6.9*sin(i*(6.28/numeng))),24.5), _V((0.08*cos(i*(6.28/numeng))),(0.08*sin(i*(6.28/numeng))), 0.95));
		AddExhaustStream (retro_engine[0], _V((6.9*cos(i*(6.28/numeng))),(6.9*sin(i*(6.28/numeng))),24.5), &contrail);
	}


/*
	th_main[0] = CreateThruster (_V(0,0.0,-15.5),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, 4e4, 4e5);

	th_main[1] = CreateThruster (_V(-3.7,0.0,-11.5), _V( 0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	th_main[2] = CreateThruster (_V( 3.7,0.0,-11.5), _V( -0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	thg_main = CreateThrusterGroup( th_main, 3, THGROUP_MAIN );
	AddExhaustStream (th_main[0], _V( 0.0,  0, -25.0), &exhaust_main);
	AddExhaustStream (th_main[1], _V(-2.6, 0.0,-20.0), &exhaust_main);
	AddExhaustStream (th_main[2], _V( 2.6, 0.0,-20.0), &exhaust_main);

	AddExhaust (th_main[0], 10.0, 1.0, tex);
	AddExhaust (th_main[1], 10.0, 0.5, tex);
	AddExhaust (th_main[2], 10.0, 0.5, tex);

	th_hover[0] = CreateThruster (_V(0,0,0), _V(0,1,0), HoverThrust, ph_main, 4e4, 4e5);
	thg_hover = CreateThrusterGroup (th_hover, 1, THGROUP_HOVER);

	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
*/
	// set of attitude thrusters (idealised). The arrangement is such that no angular
	// momentum is created in linear mode, and no linear momentum is created in rotational mode.
/*
	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);

	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);

	th_att_rot[0] = CreateThruster (_V( 2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(-2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(-2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V( 2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0,-16), _V(0,0, 1), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0, 16), _V(0,0,-1), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+1, 1, THGROUP_ATT_BACK);
*/


}


void VBS1X::TimeStep( double simt )
{

	SetAnimation (anim_expand, expand_proc);
	double da = oapiGetSimStep() * EXPAND_SPEED;
	expand_proc = expand_proc + da;
//  if (expand_proc > 1.0) expand_proc = expand_proc - 1.0;

//	ActivateNavmode (NAVMODE_KILLROT);

	if ((GetAltitude() < 300.0) && (GetSlipAngle() < 1.0))
		SetThrusterGroupLevel(thg_retro, 1.0);
	else
		SetThrusterGroupLevel(thg_retro, 0.0);


}

int VBS1X::ConsumeKey ( const char *keystate)
{
	return 0;
}

int VBS1X::ConsumeBufferedKey ( DWORD dwKey, bool fPressed, const char *keystate)
{

		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
			//	OperateDoors (door_status == DOOR_CLOSED || door_status == DOOR_CLOSING ?
			//		DOOR_OPENING : DOOR_CLOSING);
			}
			return 1;
		}
/*
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
			

				VESSELSTATUS vs;
				OBJHANDLE hvessel;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = _V(3.5,0.0,17.25);
				Local2Rel (ofs, vs.rpos);
				char name[256];
				numevas++;
				sprintf(name, "EVA-%d", numevas);
	
				VECTOR3 vel = _V(0,0.2,1.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				VESSEL::Create (name, "EVA", vs);
				hvessel=oapiGetVesselByName(name);
				oapiSetFocusObject(hvessel);
	
			}
			return 1;
		}
*/

		return 0;


}




/* basic vessel class setup */

void VBS1X::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void VBS1X::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void VBS1X::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void VBS1X::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void VBS1X::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hHawk);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx( tmp);
    Draw();
}

void VBS1X::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	ClearMeshesEx();
	COMMON_MESH tmp;
	tmp.MeshHandle(hHawk);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx( tmp);
    Draw();
}

int VBS1X::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VBS1X (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (VBS1X*)vessel;
}

