// ==============================================================
// ORBITER vessel module: von Braun Ferry
// By Erik Anderson (Sputnik)
// An adaptation of Soyuz code by Manuel Amorim (McDope)
//
// Portions taken from Rob Conley's code:
// GetCPitch(), Autopilot()
//
//
// Thanks to:
// Rob Conley (misc functions), 
// Vinka (autopilot)
//
//
// Credits:
// Core module by Manuel Amorim (McDope)
// Animation and enhancements to the meshes by Felipe Comparini (ObiWan)
// Guidance algorithm by Richard Croy (rjcroy)
// ==============================================================


#ifndef _VBSAT_H_
#include "vbSat.h"
#include <stdio.h>
#include <math.h>
#endif


// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-60*RAD,-30*RAD,-1*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
	static const double CL[nlift]  = {       0,      0,    -0.8,      0,    1.4,     2,   1.6,     0,      0};
	static const double CM[nlift]  = {       0,      0,   0.014, 0.0034,-0.0074,-0.094,-0.012,     0,      0};
	for (i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;
	// aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;
	// aoa-dependent moment coefficient
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// zero-lift drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}



VBFERRY::VBFERRY (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	stage_sep = 0;
	bManualSeparate = false;
	vis = NULL;

	hstg1 = oapiLoadMeshGlobal ("vbsatstage1");
	hstgi = oapiLoadMeshGlobal ("vbsatinterstage");
	hstg2 = oapiLoadMeshGlobal ("vbsatstage2");
	hstg3 = oapiLoadMeshGlobal ("vbsat");
//	vbPayload1 = oapiLoadMeshGlobal ("vbPayld1");

	tex     = oapiRegisterExhaustTexture ("RealExhaust");
	texRCS  = oapiRegisterExhaustTexture ("RealExhaust");

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\VB\\gear.wav",BOTHVIEW_FADED_FAR);

	N=16;
	double *fmet = new double[N];
	double *fcpitch = new double[N];

	fmet[0]					= 0.0;
	fmet[1]					= 45.0;
	fmet[2]					= 55.0;
	fmet[3]					= 65.0;
	fmet[4]					= 75.0;
	fmet[5]					= 85.0;
	fmet[6]					= 165.0;
	fmet[7]					= 175.0;
	fmet[8]					= 185.0;
	fmet[9]					= 195.0;
	fmet[10]				= 205.0;
	fmet[11]				= 215.0;
	fmet[12]				= 225.0;
	fmet[13]				= 250.0;
	fmet[14]				= 410.0;
	fmet[15]				= 430.0;

//const double cpitch[N] = {90, 85, 80, 70, 60, 50, 40, 30, 20, 17.5, 15, 20, 14, -10};	// Commanded pitch in 
	fcpitch[0]				= 89.0;
	fcpitch[1]				= 85.0;
	fcpitch[2]				= 80.0;
	fcpitch[3]				= 70.0;
	fcpitch[4]				= 60.0;
	fcpitch[5]				= 50.0;
	fcpitch[6]				= 40.0;
	fcpitch[7]				= 30.0;
	fcpitch[8]				= 20.0;
	fcpitch[9]				= 15.0;
	fcpitch[10]				= 10.0;
	fcpitch[11]				=  5.0;
	fcpitch[12]				=  2.5;
	fcpitch[13]				=  0.0;
	fcpitch[14]				=  0.0;
	fcpitch[15]				=-89.0;
	ProgramPitchMet( N, fmet, fcpitch );

	/**
	* Ear Candy
	*/
	anthem_played = false;
}

VBFERRY::~VBFERRY()
{
	CloseGuidance();
}

/*
void VBFERRY::InitPanel( int panel )
{


	switch (panel) 
	{
	case 0: // main panel
		srf[0] = oapiCreateSurface (LOADBMP (IDB_PANEL1));

	}
}
*/

void VBFERRY::InitialiseGuidance()
{
	

	bManualSeparate = false;
	stage_sep=0;
	stage = 2;
	status = 2;
	ignition= false;
	time=0.0;
	ThirdstageExists = true;

	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.6;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
	Planet_Name = new char[6];
	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
 	APO_ALT = 220 ;				// Altitude of apoapsis


	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.
//	PadHeading = 242.98;		// Heading of the Soyuz on the pad (deg)
	PadHeading = 270.00;
								// Default heading is for inclination 51.6 deg.
	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS

}

double VBFERRY::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).
void VBFERRY::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 
	
	// Only use heading control after T + 15 seconds.
	if (MET > 15 ) 
	{
		// Get state data
		GetShipAirspeedVector(Airspeed_Vector);
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));
			
		GetEquPos( Longitude, Latitude, Radius );;
		// Calculate earth rotation speed.
		Earth_x_Vel = (2*PI*(cos(fabs(Latitude)) * R_EARTH)) / 86164.09;
		Airspeed_Vector.x = Airspeed_Vector.x + Earth_x_Vel ;
		
		// Soyuz true course
		True_Course = atan(Airspeed_Vector.x / fabs(Airspeed_Vector.z)) * DEG; 
		
		//  Find the true course SP
		oapiGetHeading( this->GetHandle(), &Heading ) ;
		Clairaut_Constant = sin ( 90 * RAD - INCLINATION * RAD) ;
		
		if (True_Course < 90) 
		{
			True_Course_SP = (asin(Clairaut_Constant / cos(Latitude))) * DEG ;
		}
		else 
		{
			True_Course_SP = -((asin(Clairaut_Constant / cos(Latitude))) * DEG) + 180 ;
		}
		
		// Calculate the current orbital inclination
		oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
		oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);

		// Change ecliptic rel. vectors to equatorial vectors.
 		temp.x = Rel_Vel.x;
		temp.y = Rel_Vel.z;
		temp.z = Rel_Vel.y;
		veccpy( Rel_Vel, temp);
		temp.x = Rel_Pos.x;
		temp.y = Rel_Pos.z;
		temp.z = Rel_Pos.y;
		veccpy( Rel_Pos, temp);
		temp.x = Rel_Pos.x ;
		temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
		temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
		veccpy( Rel_Pos, temp);
		temp.x = Rel_Vel.x ;
		temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
		temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
		veccpy( Rel_Vel, temp);

		h_vector = crossp(Rel_Vel, Rel_Pos);
		h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
		i = 180 - acos(h_vector.z / h_abs) * DEG;
		i_dot = (i - i_last) / (MET - SimT_Last) ;
		i_last = i;
		ICF = Velocity / ICF_K;

		// Calculated error in inclination as output signal
		i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
		if (i_err > ICF )
		{	
			i_err = ICF;
		}
		else if ( i_err < -ICF) 
		{
			i_err = -ICF;
		}
			
		// Feed the inclination error into the control loop after T + 25s 
		if (MET > 25) {
			True_Course_SP = True_Course_SP + i_err ;
		}		
			
		True_Course_Err = True_Course - True_Course_SP; 

		// True course error rate
		TC_Error_Dot = (True_Course_Err - TC_Error_Last) / (MET - SimT_Last) ;
		TC_Error_Last = True_Course_Err;
			
		// Adjusted heading setpoint
		Heading_SP_Ajd = True_Course_Err * 4 + TC_Error_Dot * 8 ;
			
		if (Heading_SP_Ajd > 4 )
		{	
			Heading_SP_Ajd = 4;
		}
		else if (Heading_SP_Ajd < -4) 
		{
			Heading_SP_Ajd = -4;
		}
		
		//	  Compare with the actual heading
		Error_Course = Heading * DEG - ( True_Course_SP - Heading_SP_Ajd); 
		Yaw_Error_Dot =  - Angular_Velocity.y * DEG ; //get the yaw rate of change
			
		//sprintf(oapiDebugString(), "MET: %f, TrueCourse: %f, Heading: %f, Pitch: %f, Pitch SP: %f", MET, True_Course, Heading * DEG, Pitch, Pitch_SP);
			
		//  Yaw control mechanism
		Yaw_Cntrl_Sig = - YC_D * Yaw_Error_Dot + YC_K * Error_Course;
	
		//sprintf(oapiDebugString(),"MET %f, Soyuz TC %f, TrueCourseSP, %f, TrueCrsError, %f, YawSig %f, Inc, %f, i_err*4, %f, idot, %f", MET, True_Course, True_Course_SP, True_Course_Err, Yaw_Cntrl_Sig, i, i_err, i_dot);

		if (Yaw_Cntrl_Sig > 1 )
		{
			SetAttitudeRotLevel( 1, -1); // 0 is y-axis
		}
		else if (Yaw_Cntrl_Sig < -1) 
		{
			SetAttitudeRotLevel( 1, 1);
		}
		else {
			SetAttitudeRotLevel( 1 , -Yaw_Cntrl_Sig);
		}
	}
		
	// sprintf(oapiDebugString(), "Time: %f, Pitch_SP: %f, Pitch: %f, PitchErrorDot: %f, Output: %f",MET, Pitch_SP, Pitch, Pitch_Error_Dot, Pitch_Cntrl_Sig);
		
	if ( (i - INCLINATION) < 0.006 && MET > 150)
	{
		sprintf(oapiDebugString(),"T + %.0f s  Guidance Converged",MET);
	}
	else 
	{
		sprintf(oapiDebugString(),"T + %.0f s",MET);
	}

}

// rjcroy Guidance & Control System
void VBFERRY::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = oapiGetGbodyByName ( Planet_Name );


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 11;
		}

		if (MET > -5 && MET < -3) {
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
		}
		else if (MET > -3 && MET < 0) {
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
		}
		sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}
	else if ((!Alt_Flag) || (PostMECO))
	{


		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
			
		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
		
		if (Pitch_Cntrl_Sig > 1 )
		{
			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
			SetAttitudeRotLevel( 0, -1);
		}
		else {
			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
		}
		
		// Heading control
		if (MET < 408)
			Lateral_Guidance(this, SimT, SimT_Last);
		else 
			SetAttitudeRotLevel( 1, 0);
		

		// Roll control
		// Adapted from Hendo's Ariane V
		// A bit after launch, we roll inverted.  That's all.
		
		if( MET > 40.0 ) {
		double target = 180.0;
		VESSELSTATUS2 vs;
				
		vs.version = 2;
		vs.flag = 0;
		vessel->GetStatusEx(&vs);

//		diff = vs.arot.z*180/PI - target;
		double diff = vessel->GetBank()*180/PI + target;
		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		double targetrate = diff/5.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;
	
		
		if( vs.vrot.z*180/PI > targetrate ) {
			IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05 );
			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
		}
		else {
			IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05);
			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );

		// END Hendo's roll control

		}
		}
		// Turn off warp for final section of orbital insertion.
		if (MET > 403 && MET < 408)  
			oapiSetTimeAcceleration (1 );

		if (MET > 403 && oapiGetTimeAcceleration() > 1)  {
			guidance = false;
			sprintf(oapiDebugString(), "");
//			oapiSetTimeAcceleration (1 );
			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			// Turn off pitch thrusters
			SetAttitudeRotLevel( 0 , 0);
			// Turn off the yaw thrusters
			SetAttitudeRotLevel( 1, 0);


//			vessel->ActivateNavmode(NAVMODE_PROGRADE);
		}
	
		//Timeout on Alt_Flag
		
		if (MET > 408) {
			Alt_Flag = true;
			// Record the current vertical speed and time for next section.
			GetHorizonAirspeedVector( Surface_Vel );
			Vert_Vel_Last = Surface_Vel.y ;
			SimT_Last = SimT;	
		}
		
	}
	else if (!MECO_Cond)  // Final ~ 30 secs of launch
	{

		//Calculate if MECO velocity has been reached
		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);
		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));
		
		if (Velocity > V_periapsis) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
		}

		// Get vertical velocity and vertical acceleration
		GetHorizonAirspeedVector( Surface_Vel );
		Vert_Acc = (Surface_Vel.y - Vert_Vel_Last) / (MET - SimT_Last);
		Vert_Vel_Last = Surface_Vel.y;
		
		// Engine Throttling
		// Restricts acceleration in final stages to about 2.5G
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, ( oapiGetMass(hVessel) * 25 / ENG_THRST_2 ));
	
		// Pitch Control
		Pitch_SP = - D_FINAL * Vert_Acc - K_FINAL * Surface_Vel.y;

		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig = - PC_D_FINAL * Pitch_Error_Dot + PC_K_FINAL * Pitch_Error;
		
		if (Pitch_Cntrl_Sig > 1 )
		{
			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
			SetAttitudeRotLevel( 0, -1);
		}
		else {
			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
		}
		
		// Heading control
		Lateral_Guidance(this, SimT, SimT_Last);


		// sprintf(oapiDebugString(), "MET: %f, Surface_Vel.y, %f, Vert Acc: %f, PitchSP, %f, PC_DOut: %f, PC_KOut: %f", MET, Surface_Vel.y, Vert_Acc, Pitch_SP, Pitch_Error_Dot, Pitch_Error );
	}
	else if (MECO_Cond && !PostMECO)
	{
		// MECO.  Shutoff main engine, neutralize thrusters, turn off guidance
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
		RevertRadiator();
/*
		// Turn off pitch thrusters
		SetAttitudeRotLevel( 0 , 0);
		// Turn off the yaw thrusters
		SetAttitudeRotLevel( 1, 0);
		ActivateNavmode(NAVMODE_KILLROT);
*/

		// retrieve the final orbital elements
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		Alt_Periapsis = (elements.a * ( 1 - elements.e )/1000) - 6371; 
		
		// Inclination (equatorial)
		oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
		oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
		
		// Change ecliptic rel. vectors to equatorial vectors.
 		temp.x = Rel_Vel.x;
		temp.y = Rel_Vel.z;
		temp.z = Rel_Vel.y;
		veccpy( Rel_Vel, temp);
		temp.x = Rel_Pos.x;
		temp.y = Rel_Pos.z;
		temp.z = Rel_Pos.y;
		veccpy( Rel_Pos, temp);
		temp.x = Rel_Pos.x ;
		temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
		temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
		veccpy( Rel_Pos, temp);
		temp.x = Rel_Vel.x ;
		temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
		temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
		veccpy( Rel_Vel, temp);

		h_vector = crossp(Rel_Vel, Rel_Pos);
		h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
		i = 180 - acos(h_vector.z / h_abs) * DEG;
		
		PostMECO = true;

	}
/*	else if (MECO_Cond && PostMECO) 
	{
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
*/
}
// END Guidance_Routine()

// Guidance user interface
void VBFERRY::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
//	bool IncInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
}

bool VBFERRY::IncInput (void *id, char *str, void *data)
{
	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}

	return 1 ;
}


// END rjcroy Guidance System


void VBFERRY::SetAttControlsFirst()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( 0.5, 3.25,-13.95), _V(0,0.707,-0.707));
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( -0.5, 3.25,-13.95), _V(0,0.707,-0.707));
		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( 0.5,  -3.25,-13.95  ), _V(0,-0.707,-0.707));
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( -0.5, -3.25,-13.95  ), _V(0,-0.707,-0.707));
		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
//	AddExhaust (th_rot[0], 5.5, 0.15, _V(3.25, -0.5,-13.95 ), _V(0.707,0,-0.707));
//	AddExhaust (th_rot[0], 5.5, 0.15, _V(3.25, 0.5,-13.95 ), _V(0.707,0,-0.707));
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( -3.25,-0.5,-13.95 ), _V(-0.707,0,-0.707));	
//	AddExhaust (th_rot[0], 5.5, 0.15, _V( -3.25, 0.5,-13.95 ), _V(-0.707,0,-0.707));
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);
	th_rot[0] = CreateThruster (_V(3,0,0), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);
	th_rot[0] = CreateThruster (_V(3,0, 0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0, -1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);
}

void VBFERRY::SetAttControlsSecond()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 11), _V(0, 1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(0,0,-11), _V(0,-1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);
	th_rot[0] = CreateThruster (_V(0,0, 11), _V(0,-1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(0,0,-11), _V(0, 1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);
	th_rot[0] = CreateThruster (_V( 3,0,0), _V(0, 1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0,-1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);
	th_rot[0] = CreateThruster (_V( 3,0,0), _V(0,-1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0, 1,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_SECOND, ph_second, ISP_SECOND_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);
}

void VBFERRY::SetAttControlsThird()
{
	if (!ThirdstageExists) return;
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,-0.5, 6.5), _V(0, 1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V(0, 3.0,-6.5), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);
		thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);
	th_rot[0] = CreateThruster (_V(0, 0.5, 6.5), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V(0,-3.0,-6.5), _V(0, 1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);
		thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);
	th_rot[0] = CreateThruster (_V( 3,0,0), _V(0, 1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);
	th_rot[0] = CreateThruster (_V( 3,0,0), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V(-3,0,0), _V(0, 1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);	
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);
	th_rot[0] = CreateThruster (_V(-0.5,0, 6.5), _V( 1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V( 3.0,0,-6.5), _V(-1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);	
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);
	th_rot[0] = CreateThruster (_V( 0.5,0, 6.5), _V(-1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
	th_rot[1] = CreateThruster (_V(-3.0,0,-6.5), _V( 1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_SL);
//		AddExhaust (th_rot[0], 1.5, 0.025, texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, texRCS);
		CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 1.5, 0.025, _V(0, -3.0,-6.5), _V(0,-1.0,0), texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, _V(0, -0.5, 6.5), _V(0,-1.0,0), texRCS);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 1.5, 0.025, _V(0,  3.0,-6.5), _V(0,1.0,0), texRCS);
//		AddExhaust (th_rot[1], 1.5, 0.025, _V(0,  0.5, 6.5), _V(0,1.0,0), texRCS);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V( 3.0,  0,-6.5), _V(1.0,0,0), texRCS);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V( 0.5,  0, 6.5), _V(1.0,0,0), texRCS);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0), MAX_ATT_THIRD , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V(-3.0,  0,-6.5), _V(-1.0,0,0), texRCS);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V(-0.5,  0, 6.5), _V(-1.0,0,0), texRCS);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);
	th_rot[0] = CreateThruster (_V(0,0, -6.7), _V(0,0,1), MAX_ATT_THIRD*4 , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 3.0, 0.05, tex);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0,-6.7), _V(0,0,-1), MAX_ATT_THIRD*4 , ph_third, ISP_THIRD_VAC);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V( 3.0,  0, 3.0), _V(0,0,1), texRCS);
//		AddExhaust (th_rot[0], 1.25, 0.025, _V(-3.0,  0, 3.0), _V(0,0,1), texRCS);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);

	// **************************** aerodynamics ***********************************


}

void VBFERRY::DefineAnimations ()
{
	if (!ThirdstageExists) return;

	static UINT AntennaGrp[4] = {9, 10, 11, 12};
	static MGROUP_SCALE Antenna (0, AntennaGrp, 4,
		_V(0,0,0), _V(20,20,1));

	static UINT BoilerGrp[4] = {1, 7, 13, 14};
	static MGROUP_TRANSLATE Boiler (0, BoilerGrp, 4,
		_V(0,0,-6.0));

	static UINT Boiler2Grp[2] = {1, 7};
	static MGROUP_ROTATE Boiler2 (0, Boiler2Grp, 2,
		_V(0,0,-6.5), _V(0,1,0), (float)(-90.0*RAD));

	static UINT Boiler3Grp[1] = {1};
	static MGROUP_SCALE Boiler3 (0, Boiler3Grp, 1,
		_V(0,0,0), _V(20,20,1));


// Known Groups:
// 0  Mainbody
// 1  Mirror
// 2  RedRing
// 3  Nozl1
// 4  Nozl2
// 5  Nozl3
// 6  Nozl4
// 7  Boiler
// 8  YelloCone
// 9  Ant 1
// 10 Ant 2
// 11 Ant 3
// 12 Ant 4
// 13 LargerPole
// 14 SmalerPole

	anim_radiator = CreateAnimation (1);

	AddAnimationComponent (anim_radiator, 0.05, 0.9, &Antenna);
	AddAnimationComponent (anim_radiator, 0.05, 0.79, &Boiler);
	AddAnimationComponent (anim_radiator, 0.81, 0.95, &Boiler2);
//	AddAnimationComponent (anim_radiator, 0.5, 0.79, &Boiler3);

}

void VBFERRY::UpdateAnimations()
{	
	// animate radiator
	if (radiator_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * RADIATOR_OPERATING_SPEED;
		if (radiator_status == DOOR_CLOSING) {
			if (radiator_proc > 0.0)
				radiator_proc = max (0.0, radiator_proc-da);
			else {
				radiator_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (radiator_proc < 1.0)
				radiator_proc = min (1.0, radiator_proc+da);
			else {
				radiator_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_radiator, radiator_proc);
	}

}

void VBFERRY::ActivateRadiator (DoorStatus action)
{
	radiator_status = action;
}

void VBFERRY::RevertRadiator ()
{
	ActivateRadiator (radiator_status == DOOR_CLOSED || radiator_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
//	StopVesselWave3(MyID,PLAYGEAR);
//	PlayVesselWave3(MyID,PLAYGEAR);
}


void VBFERRY::SetFirstStage ()
{
	PARTICLESTREAMSPEC contrail = {
		0, 12.0, 5, 150, 0.3, 4.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_main = {
		0, 11.0, 20, 150, 0.1, 0.2, 16, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");


	if (GetSize() == 40) return;
	SetSize (40);
	SetCOG_elev (9);
	double tempmass;
	tempmass = STAGE1_EMASS+STAGE2_EMASS+STAGE2_PMASS;
	if (ThirdstageExists) tempmass += STAGE3_EMASS+STAGE3_PMASS;
	SetEmptyMass (tempmass+CalculateMass());
	SetPMI (_V(25,25,15));
	SetCrossSections (_V(20,20,10));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	ClearMeshes();
//	SetTouchdownPoints (_V(0,-1.0,-5.0),  _V(-.7,.7,-5.0),  _V(.7,.7,-5.0));
	SetTouchdownPoints (_V(0,-1.0,-25.4), _V(-.7,.7,-25.4), _V(.7,.7,-25.4));
//	ShiftCentreOfMass (_V(0,0,72.0));

	COMMON_MESH tmp;
	ClearMeshesEx();
	tmp.MeshHandle(hstg3);
	tmp.MeshPosition(OFS_STAGE3);
	if (ThirdstageExists) AddMeshEx (tmp);
	tmp.MeshHandle(hstg2);
	tmp.MeshPosition(OFS_STAGE2);
	AddMeshEx (tmp);
	tmp.MeshHandle(hstgi);
	tmp.MeshPosition(OFS_STAGEi);
	AddMeshEx (tmp);
	tmp.MeshHandle(hstg1);
	tmp.MeshPosition(OFS_STAGE1);
	AddMeshEx (tmp);
	Draw();

//	if (ThirdstageExists) AddMesh (hstg3, &OFS_STAGE3);
//	AddMesh (hstg2, &OFS_STAGE2);
//	AddMesh (hstg1, &OFS_STAGE1);

//    InitNavRadios (2);
	VECTOR3 mesh_dir=_V(0,0.850,21.5);
	SetCameraOffset(mesh_dir);
	if (!ph_first)  ph_first  = CreatePropellantResource (STAGE1_PMASS); 
	SetDefaultPropellantResource (ph_first); 
	ClearThrusterDefinitions();
	th_first[0] = CreateThruster (_V(4.5,0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(0,0,-30.8), &contrail);
	AddExhaustStream (th_first[0], _V(0,0,-30.8), &exhaust_main);
	th_first[1] = CreateThruster (_V(-4.5,0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[2] = CreateThruster (_V(0,4.5,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[3] = CreateThruster (_V(0,-4.5,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[4] = CreateThruster (_V(-2.0,2.0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[5] = CreateThruster (_V(2.0,2.0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[6] = CreateThruster (_V(2.0,-2.0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	th_first[7] = CreateThruster (_V(-2.0,-2.0,-24.4), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	thg_first = CreateThrusterGroup (th_first, 8, THGROUP_MAIN);
	for (int i = 0; i < 8; i++) AddExhaust (th_first[i], 30.0, 3.5, tex);
	SetAttControlsFirst();
	status = 0;
}

void VBFERRY::SetSecondStage ()
{
	PARTICLESTREAMSPEC contrail = {
		0, 5.5, 5, 150, 0.3, 4.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_main = {
		0, 5.0, 20, 150, 0.1, 0.2, 16, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");


	if (GetSize() == 22) return;
	SetSize (22);
//	SetCOG_elev (52.4);
	double tempmass;
	tempmass = STAGE2_EMASS;
	if (ThirdstageExists) tempmass += STAGE3_EMASS+STAGE3_PMASS;
	SetEmptyMass (tempmass+CalculateMass());
	SetPMI (_V( 5, 5, 3));
	SetTouchdownPoints (_V(0,-1.0,-22.4), _V(-.7,.7,-22.4), _V(.7,.7,-22.4));
	SetCrossSections (_V(105,105,25));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	ClearMeshes();
	ShiftCentreOfMass (_V(0,0,0));

	COMMON_MESH tmp;
	ClearMeshesEx();
	tmp.MeshHandle(hstg3);
	tmp.MeshPosition(OFS_STAGE3);
	if (ThirdstageExists) AddMeshEx (tmp);
	tmp.MeshHandle(hstg2);
	tmp.MeshPosition(OFS_STAGE2);
	AddMeshEx (tmp);
	Draw();

//	if (ThirdstageExists) AddMesh (hstg3, &OFS_STAGE3);
//	AddMesh (hstg2, &OFS_STAGE2);
//	InitNavRadios (2);
	VECTOR3 mesh_dir=_V(0,0.850,21.5);
	SetCameraOffset(mesh_dir);
	ClearThrusterDefinitions();
	if (ph_first) DelPropellantResource (ph_first);
	if (!ph_second) ph_second  = CreatePropellantResource (STAGE2_PMASS); 
	SetDefaultPropellantResource (ph_second); 

	th_second[0] = CreateThruster (_V(2.1,0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_second[0], _V(0,0,-8.25), &contrail);
	AddExhaustStream (th_second[0], _V(0,0,-8.25), &exhaust_main);
	th_second[1] = CreateThruster (_V(-2.1,0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[2] = CreateThruster (_V(0,2.1,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[3] = CreateThruster (_V(0,-2.1,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[4] = CreateThruster (_V(-1.0,1.0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[5] = CreateThruster (_V(1.0,1.0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[6] = CreateThruster (_V(1.0,-1.0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	th_second[7] = CreateThruster (_V(-1.0,-1.0,-5.75), _V(0,0,1), THRUST_SECOND_VAC, ph_second, ISP_SECOND_VAC, ISP_SECOND_SL);
	thg_second = CreateThrusterGroup (th_second, 8, THGROUP_MAIN);
	for (int i = 0; i < 8; i++) AddExhaust (th_second[i], 15.0, 1.75, tex);
	SetThrusterGroupLevel (thg_second,1.0);
	SetAttControlsSecond();
	status = 1;
}


void VBFERRY::SetThirdStage ()
{

	if (GetSize() == 15) return;
	SetSize (15);
	SetCOG_elev (0);
	if (People)
		SetEmptyMass (STAGE3_EMASS+PEOPLEMASS+PAYLOADMASS);
	else SetEmptyMass (STAGE3_EMASS+PAYLOADMASS);
	SetPMI (_V(23.75,23.75,2.25));
	SetCrossSections (_V(35,65,15));
//	SetTouchdownPoints (_V(0,-4.2,14.0), _V(-9.15,-4.6,-3.0), _V(9.15,-4.6,-3.0));
	BrakesOn = false;
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetTrimScale (0.05);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetWingAspect(10.0);
	SetWingEffectiveness(2.8);
//	SetLiftCoeffFunc (LiftCoeff); 
	VECTOR3 dockpos = {0.0,0.0,14.0};
	VECTOR3 dockdir = {0,0,1};
	VECTOR3 dockrot = {0,1,0};
	SetDockParams (dockpos, dockdir, dockrot);
	ClearMeshes();
	VECTOR3 mesh_dir=_V(0,0,0);

	COMMON_MESH tmp;
	ClearMeshesEx();
	tmp.MeshHandle(hstg3);
	tmp.MeshPosition(mesh_dir);
	AddMeshEx (tmp);
	Draw();

//	AddMesh (hstg3, &mesh_dir);
//	VECTOR3 payload_dir=_V(0.0, 0.350, 8.60);
//	AddMesh (vbPayload1, &payload_dir);
	ShiftCentreOfMass (_V(0,0,20.1));
//	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	mesh_dir=_V(0,0.850,11.4);
	SetCameraOffset(mesh_dir);
//    InitNavRadios (2);

	if (ph_second) DelPropellantResource (ph_second);
	if (!ph_third) ph_third  = CreatePropellantResource (STAGE3_PMASS);
	SetDefaultPropellantResource (ph_third); 

	ph_H2O = CreatePropellantResource (H2OMASS);
	SetPropellantMass(ph_H2O,H2OMASS*H2O);

	ClearThrusterDefinitions();
	th_third[0] = CreateThruster (_V(1.05,0,-3.75), _V(0,0,1), THRUST_THIRD_VAC, ph_third, ISP_THIRD_VAC, ISP_THIRD_SL);
	th_third[1] = CreateThruster (_V(-1.05,0,-3.75), _V(0,0,1), THRUST_THIRD_VAC, ph_third, ISP_THIRD_VAC, ISP_THIRD_SL);
	th_third[2] = CreateThruster (_V(0,1.05,-3.75), _V(0,0,1), THRUST_THIRD_VAC, ph_third, ISP_THIRD_VAC, ISP_THIRD_SL);
	th_third[3] = CreateThruster (_V(0,-1.05,-3.75), _V(0,0,1), THRUST_THIRD_VAC, ph_third, ISP_THIRD_VAC, ISP_THIRD_SL);
	thg_third = CreateThrusterGroup (th_third, 4, THGROUP_MAIN);
	for (int i = 0; i < 4; i++)	AddExhaust (th_third[i], 5.0, 1.0, tex);
	SetThrusterGroupLevel (thg_third,1.0);
	SetAttControlsThird();
	status = 2;    
}


void VBFERRY::SeparateStage (UINT stage)
{
	VESSELSTATUS vs;
	char name[256], name2[256];
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	GetStatus (vs);
    vs.eng_main = vs.eng_hovr = 0.0;

	if (stage == 0)
	{
		ofs = OFS_STAGE1;
		vel = _V(0,0,-2.0);	
	}

	else if (stage == 1)
	{
		ofs = OFS_STAGE2;	
		vel = _V(0,0,-2.0);	
	}
	else if (stage == 2)
	{
		ofs = _V(0.0, 0.350, 8.60);		
		vel = _V(0.0,0.1,0.0); 
	}

	
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	
	if (stage == 0)
	{
		vs.vrot.x = 0.0;	
		vs.vrot.y = 0.0;
		vs.vrot.z = 0.0;

		vs.fuel = 1.0;

		strcpy (name2, GetName()); 
		sprintf(name, " First Stage");
		strcat (name2, name);
		oapiCreateVessel(name2, "vbSatS1", vs);

		ofs = OFS_STAGEi;
		vel = _V(0,0,-1.0);	
		GetStatus (vs);
		VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
		Local2Rel (ofs, vs.rpos);
		GlobalRot (vel, rofs);
		vs.rvel.x = rvel.x+rofs.x;
		vs.rvel.y = rvel.y+rofs.y;
		vs.rvel.z = rvel.z+rofs.z;
		vs.vrot.x = 0.0;	
		vs.vrot.y = 0.0;
		vs.vrot.z = 0.0;


		strcpy (name2, GetName()); 
		sprintf(name, " Interstage");
		strcat (name2, name);
		oapiCreateVessel(name2, "vbSatInt", vs);

		DelPropellantResource (ph_first);

		DelThrusterGroup (thg_first, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

		ClearThrusterDefinitions();

		SetSecondStage ();	
	}

    else if ((stage == 1) && (ThirdstageExists))
	{
	    vs.vrot.x = 0.0;	
		vs.vrot.y = 0.0;
		vs.vrot.z = 0.0;

		vs.fuel = 1.0;

		strcpy (name2, GetName()); 
		sprintf(name, " Second Stage");
		strcat (name2, name);
		oapiCreateVessel(name2, "vbSatS2", vs);
//		oapiCreateVessel("Ferry Second Stage", "vbFerryS2x", vs);

		DelPropellantResource (ph_second);

		DelThrusterGroup (thg_second, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

		ClearThrusterDefinitions();

		SetThirdStage ();	
	}

    else if ((stage == 1) && (!ThirdstageExists))
	{

		depV = DeployPayload(_V(0,0,0));
		status = 2;
		COMMON_MESH tmp;
		ClearMeshesEx();
		tmp.MeshHandle(hstg3);
		tmp.MeshPosition(OFS_STAGE3);
		if (ThirdstageExists) AddMeshEx (tmp);
		tmp.MeshHandle(hstg2);
		tmp.MeshPosition(OFS_STAGE2);
		AddMeshEx (tmp);
		Draw();
//		depV->SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		hVessel=oapiGetVesselByName(depV->GetName());
		if (hVessel != 0)
			oapiSetFocusObject(hVessel);


/*		DelPropellantResource (ph_second);

		DelThrusterGroup (thg_second, THGROUP_MAIN, true);
		DelThrusterGroup (thg_pitchup, THGROUP_ATT_PITCHUP, true);
		DelThrusterGroup (thg_pitchdown, THGROUP_ATT_PITCHDOWN, true);

		DelThrusterGroup (THGROUP_ATT_BANKLEFT, true);
		DelThrusterGroup (THGROUP_ATT_BANKRIGHT, true);
		DelThrusterGroup (THGROUP_ATT_YAWLEFT, true);
		DelThrusterGroup (THGROUP_ATT_YAWRIGHT, true);

		ClearThrusterDefinitions();
		SetSize(20);
		SetSecondStage (vessel);	
*/
	}

	else if (stage == 2)
	{
		vs.vrot.x = 0.0;	
		vs.vrot.y = 0.0;	
		vs.vrot.z = 0.0;	
//		oapiCreateVessel("Payload", "multistage/maqsat", vs);



		
//		Payload = false;

		oapiCreateVessel ("vbPayload1", "vbPayload1", vs);
		
		SetThirdStage ();
		
	}
}
/*
void VBFERRY::RedrawPanel_LED_A (SURFHANDLE surf)
{
	char cbuf[16];
	HDC hDC = oapiGetDC (surf);
	SelectObject (hDC, g_Param.font[0]);
	SetTextColor (hDC, RGB(255, 0, 0));
	SetTextAlign (hDC, TA_RIGHT);
	SetBkMode (hDC, TRANSPARENT);
	SelectObject (hDC, g_Param.brush[0]);
	SelectObject (hDC, GetStockObject (NULL_PEN));

	sprintf (cbuf, "Pitch %0.2lf", Pitch );
	TextOut (hDC, 72, 5, cbuf, strlen(cbuf));
	oapiReleaseDC (surf, hDC);
}

void VBFERRY::RedrawPanel_LED_B (SURFHANDLE surf)
{
	char cbuf[16];
	HDC hDC = oapiGetDC (surf);
	SelectObject (hDC, g_Param.font[0]);
	SetTextColor (hDC, RGB(255, 0, 0));
	SetTextAlign (hDC, TA_RIGHT);
	SetBkMode (hDC, TRANSPARENT);
	SelectObject (hDC, g_Param.brush[0]);
	SelectObject (hDC, GetStockObject (NULL_PEN));

	sprintf (cbuf, "MET %0.2lf", MET );
	TextOut (hDC, 72, 5, cbuf, strlen(cbuf));
	oapiReleaseDC (surf, hDC);

}
*/
void VBFERRY::TimeStep( double simt )
{

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
	}

	if (ThirdstageExists)
		UpdateAnimations();

	/*	if ( !anthem_played )
	{
		PlayVesselWave(1,NOLOOP,255,NULL);
		anthem_played = true;
	}
*/


	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
// Originally 258.40 to 133.44, changed to allow 270 launch hdg.
// This is BAD for error-checking
			if (PadHeading < 358.40 && PadHeading > 133.44) {
				Clairaut_Constant = sin((PadHeading - 180)*RAD) * cos(Latitude);
				INCLINATION = acos(Clairaut_Constant)*DEG;
			} else {
				BadHeading = true ;
			}
			
		}
	}

	// Check if we're docked.  If we are, delete the mass of the people (passengers)

	if ((DockingStatus(0) == 1) && (People = true))
	{
		People = false;
		if (stage == 2)
			SetEmptyMass (STAGE3_EMASS+PAYLOADMASS);
		else if (stage == 3)
			SetEmptyMass (STAGE3_EMASS);
	}
		
	//Check if guidance is on and if so, call
	if (guidance)
	{

		if (BadHeading) 
		{
			sprintf(oapiDebugString(),"Guidance system cannot launch from this heading.");
			guidance = false;
		} 
		
		
		else if (stage < 4) 
		{
			
			
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -10 ;
				TZero = 0;
			}

			
			else 
			{
				MET = simt - TZero;
			}

			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		
		}
	}


	// Display the inclination setpoint if the switch is on.
	if (DisplayIncSP) {
		sprintf(oapiDebugString(),"Inclination Setpoint: %.2f deg", INCLINATION);
	}
	

	if (stage == 0)
	{
    	if (GetEngineLevel(ENGINE_MAIN) > .95 && !engineignited)
		{
			ignition_time = simt;
			engineignited = true;
		}

		if (GetPropellantMass(ph_first) ==0 || bManualSeparate)
		{
			time=simt;
			SeparateStage (stage);
			bManualSeparate=false;
			stage = 1;
//			SetAnimState (anim_gear, gear_proc);
//			SetAnimState (anim_pldoors, pldoors_proc);
		} 	
	}	
	else if (stage == 1)
	{
		last_simt = simt;

		if (GetPropellantMass(ph_second) == 0 ||bManualSeparate)
		{
			time=simt;
			bManualSeparate=false;
			SeparateStage (stage);
			stage = 2;
//			SetAnimState (anim_gear, gear_proc);
//			SetAnimState (anim_pldoors, pldoors_proc);
		} 
		
	}
	else if (stage == 2)
	{
		last_simt = simt;

		if (bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (stage);
			stage = 3;
//			SetAnimState (anim_gear, gear_proc);
//			SetAnimState (anim_pldoors, pldoors_proc);
		}
	}
    else if (stage == 3)
	{
		last_simt = simt;
		
		if (bManualSeparate)
		{
			stage_sep = simt;
			bManualSeparate=false;
			SeparateStage (stage);
			stage = 4;
//			SetAnimState (anim_gear, gear_proc);
//			SetAnimState (anim_pldoors, pldoors_proc);
		}
	}

//	oapiTriggerPanelRedrawArea (0, AID_LEDA );
//	oapiTriggerPanelRedrawArea (0, AID_LEDB );
}

void VBFERRY::SetClassCaps( FILEHANDLE cfg )
{

	// panel animation
	radiator_proc        = 0;

	DefineAnimations();

//	pitchrate = 0.0;

	// propellant resources
	ph_first          = NULL;
	ph_second         = NULL;
	ph_third		 = NULL;
	thg_first        = NULL;
	thg_second       = NULL;
	thg_third		 = NULL;
	thg_pitchup		 = NULL;
	thg_pitchdown	 = NULL;
//	have_sound = ConnectToOrbiterSoundDLL();
//	if( have_sound )
//	{
//		RequestLoadVesselWave(1,"Sound\\CCCP\\sovnat1.wav");
		
//	}

}

void VBFERRY::LoadState(FILEHANDLE scn , void *vs)
{

    char *line;
	double temp;

//	InitNavRadios (2);

	while (oapiReadScenario_nextline (scn, line)) {

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */

		ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);

	

        if (!strnicmp (line, "CONFIGURATION", 13)) {
            sscanf (line+13, "%d", &status);
		} else if (!strnicmp (line, "TIME", 4)) {
            sscanf (line+4, "%lf", &temp);
			ignition_time=-temp;
			} else if (!strnicmp (line, "RADIATOR", 8)) {
		sscanf (line+8, "%d%lf", &radiator_status, &radiator_proc);
		} else if (!strnicmp (line, "H2O", 3)) {
		sscanf (line+3, "%lf", &H2O);
		} else if (!strnicmp (line, "PEOPLE", 6)) {
		sscanf (line+6, "%d", &People);
		} else if (!strnicmp (line, "THIRDSTG", 8)) {
		sscanf (line+8, "%d", &ThirdstageExists);
//		} else if (!strnicmp (line, "NAVFREQ", 7)) {
//		sscanf (line+7, "%d", &Navfreq1);
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
			double d_heading;
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading;  // It's not pad heading any more; just target heading
//			at5->guidance = true;
//		} else if (!strnicmp (line, "HEADING", 7)) {
//			sscanf(line+7, "%lf", &PadHeading);
//			ParseScenarioLineEx (line, vs);
		}
		else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
    }
	
//	SetNavRecv(0,Navfreq1);
//	SetNavRecv(1,Navfreq2);

	switch (status) {
	case 0:
		stage=0;
		SetFirstStage();
		break;
	case 1:
		stage=1;
		SetSecondStage();
		break;
	case 2:
		stage=2;
		SetThirdStage();
		break;
	case 3:
		stage=3;
		SetThirdStage();
		break;

	}
	if (ThirdstageExists) {
			SetAnimState (anim_radiator, radiator_proc);
// If the gear starts UP, then throw it down and hit RevertLandingGear to throw it back up again
// I don't know why, but if this bit isn't in here, the sim will ALWAYS start with the gear showing down
			if (radiator_proc<0.05) {
				radiator_status=DOOR_OPENING;
				RevertRadiator();
			}
			UpdateAnimations();
	}

}

void VBFERRY::SaveState( FILEHANDLE scn )
{
	char cbuf[256];
	// default vessel parameters
	SaveDefaultState (scn);
	// custom parameters
	oapiWriteScenario_int (scn, "CONFIGURATION", status);
	sprintf (cbuf, "%d %0.4f", radiator_status, radiator_proc);
	oapiWriteScenario_string (scn, "RADIATOR", cbuf);
	sprintf (cbuf, "%0.4f", H2O);
	oapiWriteScenario_string (scn, "H2O", cbuf);
	sprintf (cbuf, "%d", People);
	oapiWriteScenario_string (scn, "PEOPLE", cbuf);
	sprintf (cbuf, "%d", ThirdstageExists);
	oapiWriteScenario_string (scn, "THIRDSTG", cbuf);

	oapiWriteScenario_float (scn, "TIME", last_simt-ignition_time);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void VBFERRY::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	switch (status) {
	case 0:
		stage=0;
		SetFirstStage();
		break;
	case 1:
		stage=1;
		SetSecondStage();
		break;
	case 2:
		stage=2;
		SetThirdStage();
		break;
	case 3:
		stage=3;
		SetThirdStage();
		break;

	}
//	SetAnimState (anim_gear, gear_proc);
//	SetAnimState (anim_pldoors, pldoors_proc);
// If the gear starts UP, then throw it down and hit RevertLandingGear to throw it back up again
// I don't know why, but if this bit isn't in here, the sim will ALWAYS start with the gear showing down
	if (radiator_proc<0.05) {
		radiator_status=DOOR_OPENING;
		RevertRadiator();
	}
//	UpdateAnimations(vessel);

}

/*
bool VBFERRY::LoadPanel(VESSEL *vessel, int id)
{

	HBITMAP hBmp = LoadBitmap ( g_Param.hDLL, MAKEINTRESOURCE (IDB_PANEL1+id));
//	MFDSPEC mfds_left  = {{37, 271, 117, 451}, 6, 6, 47, 41};
//	MFDSPEC mfds_right = {{426, 271, 516, 451}, 6, 6, 47, 41};

	switch (id) {
	case 0: // main panel
		oapiRegisterPanelBackground (hBmp, PANEL_ATTACH_BOTTOM, 0xffffff);
		oapiSetPanelNeighbours (-1,-1,-1,-1);
	//	oapiRegisterMFD (MFD_LEFT, mfds_left);
	//	oapiRegisterMFD (MFD_RIGHT, mfds_right);
		oapiRegisterPanelArea (AID_LEDA,  _R(  420,  215,  513, 233 ), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (AID_LEDB,  _R(  515,  215,  611, 233 ), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (AID_BDS1,  _R(  535,  253,  595, 275 ), PANEL_REDRAW_ALWAYS, PANEL_REDRAW_MOUSE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (AID_BDS2,  _R(  535,  282,  595, 307 ), PANEL_REDRAW_ALWAYS, PANEL_REDRAW_MOUSE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (AID_BDS3,  _R(  535,  312,  595, 336 ), PANEL_REDRAW_ALWAYS, PANEL_REDRAW_MOUSE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (AID_BDS4,  _R(  535,  341,  595, 366 ), PANEL_REDRAW_ALWAYS, PANEL_REDRAW_MOUSE, PANEL_MAP_BACKGROUND);

	}

	InitPanel (id);
	return hBmp != NULL;


}

bool VBFERRY::PanelMouseEvent( VESSEL *vessel, int id, int event, int mx, int my )
{
	switch( id )
	{

	case AID_BDS1 :
		return true;
		break;
	case AID_BDS2 :
		return true;
		break;
	case AID_BDS3 :
		return true;
		break;
	case AID_BDS4 :
		return true;
		break;
	default:
		return true;
	}
 return true;
}

bool VBFERRY::PanelRedrawEvent(VESSEL *vessel, int id, int event, SURFHANDLE surf)
{
	

	switch (id) 
	{
	// panel 0 events:
	case AID_LEDA:
		RedrawPanel_LED_A  (surf);
		return true;
	case AID_LEDB:
		RedrawPanel_LED_B (surf );
		return true;
	}
	return true;
}

void VBFERRY::UpdatePanels( VESSEL *vessel,VISHANDLE visual, int refcount )
{

	vis = visual;
	UpdatePanels(vessel);

}
*/
void VBFERRY::ReleaseSurfaces ()
{
	for (int i = 0; i < nsurf; i++)
		if (srf[i]) {
			oapiDestroySurface (srf[i]);
			srf[i] = 0;
		}
}


int VBFERRY::ConsumeBufferedKey( DWORD dwKey, bool fPressed, const char *keystate)
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
		if (KEYDOWN (keystate, OAPI_KEY_U))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
				// Report the inclination setpoint
				if(DisplayIncSP) {
					DisplayIncSP = false;
				}
				else {
					DisplayIncSP = true;
				}
																
			return 1;
		}
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
					ManualSeparate (true);
			}
			return 1;
		}
/*		
		if (KEYDOWN (keystate, OAPI_KEY_K) && stage == 4) { 
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
//				OperatePanels();
			}
			return 1;
		}
*/
		if (KEYDOWN (keystate, OAPI_KEY_O))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel( 0, 0);
					SetAttitudeRotLevel( 1, 0);
					SetAttitudeRotLevel( 2, 0);
					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
		
		if (KEYDOWN (keystate, OAPI_KEY_K) && stage > 1)
		{
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
				// Toggle the Gear
				RevertRadiator ();
			return 1;
		}

		if (KEYDOWN (keystate, OAPI_KEY_U))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
				// Get a new Inclination setpoint from pilot.
				if(!guidance) {
					GetInclinationSP() ;
				}
				
			return 1;
		}
	}
	return 0;
}


// ==============================================================
// API interface
// ==============================================================




/* basic vessel class setup */

void VBFERRY::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void VBFERRY::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void VBFERRY::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void VBFERRY::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void VBFERRY::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	switch (status) {
	case 0:
		stage=0;
		SetFirstStage();
		break;
	case 1:
		stage=1;
		SetSecondStage();
		break;
	case 2:
		stage=2;
		SetThirdStage();
		break;
	case 3:
		stage=3;
		SetThirdStage();
		break;

	}
}

void VBFERRY::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	switch (status) {
	case 0:
		stage=0;
		SetFirstStage();
		break;
	case 1:
		stage=1;
		SetSecondStage();
		break;
	case 2:
		stage=2;
		SetThirdStage();
		break;
	case 3:
		stage=3;
		SetThirdStage();
		break;

	}
}

int VBFERRY::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VBFERRY (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (VBFERRY*)vessel;
}

